/*
 * $Id: PublishTransformer.java,v 1.5 2006/11/05 05:54:49 fortuna Exp $
 *
 * Created: 26/09/2004
 *
 * Copyright (c) 2004, Ben Fortuna
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  o Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 *  o Neither the name of Ben Fortuna nor the names of any other contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.fortuna.ical4j.transform;

import java.util.Iterator;
import net.fortuna.ical4j.model.Calendar;
import net.fortuna.ical4j.model.Component;
import net.fortuna.ical4j.model.Property;
import net.fortuna.ical4j.model.PropertyList;
import net.fortuna.ical4j.model.property.Method;
import net.fortuna.ical4j.model.property.Sequence;

/**
 * Transforms a calendar for publishing.
 * @author benfortuna
 */
public class PublishTransformer extends Transformer {

    /*
     * (non-Javadoc)
     * @see net.fortuna.ical4j.transform.Transformer#transform(net.fortuna.ical4j.model.Calendar)
     */
    public final Calendar transform(final Calendar calendar) {
        PropertyList calProps = calendar.getProperties();

        Property method = calProps.getProperty(Property.METHOD);

        if (method != null) {
            calProps.remove(method);
        }

        calProps.add(Method.PUBLISH);

        // if a calendar component has already been published previously
        // update the sequence number..
        for (Iterator i = calendar.getComponents().iterator(); i.hasNext();) {
            Component component = (Component) i.next();

            PropertyList compProps = component.getProperties();

            Sequence sequence = (Sequence) compProps
                    .getProperty(Property.SEQUENCE);

            if (sequence == null) {
                compProps.add(new Sequence(0));
            }
            else {
                compProps.remove(sequence);
                compProps.add(new Sequence(sequence.getSequenceNo() + 1));
            }
        }

        return calendar;
    }

}
